@@ -19,6 +19,7 @@ package admin
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -36,8 +37,11 @@ type Functions interface {
 	// CreateFunc create a new function.
 	CreateFunc(data *utils.FunctionConfig, fileName string) error
 
-	// CreateFuncWithURL create a new function by providing url from which fun-pkg can be downloaded.
-	// supported url: http/file
+	// CreateFuncWithContext create a new function.
+	CreateFuncWithContext(ctx context.Context, data *utils.FunctionConfig, fileName string) error
+
+	// CreateFuncWithURL creates a new function by providing url from which fun-pkg can be downloaded.
+	// Supported url: http/file
 	// eg:
 	//  File: file:/dir/fileName.jar
 	//  Http: http://www.repo.com/fileName.jar
@@ -48,15 +52,38 @@ type Functions interface {
 	//      url from which pkg can be downloaded
 	CreateFuncWithURL(data *utils.FunctionConfig, pkgURL string) error
 
-	// StopFunction stop all function instances
+	// CreateFuncWithURLWithContext creates a new function by providing url from which fun-pkg can be downloaded.
+	// Supported url: http/file
+	// eg:
+	//  File: file:/dir/fileName.jar
+	//  Http: http://www.repo.com/fileName.jar
+	//
+	// @param ctx
+	//        context used for the request
+	// @param functionConfig
+	//      the function configuration object
+	// @param pkgURL
+	//      url from which pkg can be downloaded
+	CreateFuncWithURLWithContext(ctx context.Context, data *utils.FunctionConfig, pkgURL string) error
+
+	// StopFunction stops all function instances
 	StopFunction(tenant, namespace, name string) error
 
-	// StopFunctionWithID stop function instance
+	// StopFunctionWithContext stops all function instances
+	StopFunctionWithContext(ctx context.Context, tenant, namespace, name string) error
+
+	// StopFunctionWithID stops function instance
 	StopFunctionWithID(tenant, namespace, name string, instanceID int) error
 
-	// DeleteFunction delete an existing function
+	// StopFunctionWithIDWithContext stops function instance
+	StopFunctionWithIDWithContext(ctx context.Context, tenant, namespace, name string, instanceID int) error
+
+	// DeleteFunction deletes an existing function
 	DeleteFunction(tenant, namespace, name string) error
 
+	// DeleteFunctionWithContext deletes an existing function
+	DeleteFunctionWithContext(ctx context.Context, tenant, namespace, name string) error
+
 	// Download Function Code
 	// @param destinationFile
 	//        file where data should be downloaded to
@@ -64,6 +91,15 @@ type Functions interface {
 	//        Path where data is located
 	DownloadFunction(path, destinationFile string) error
 
+	// Download Function Code
+	// @param ctx
+	//        context used for the request
+	// @param destinationFile
+	//        file where data should be downloaded to
+	// @param path
+	//        Path where data is located
+	DownloadFunctionWithContext(ctx context.Context, path, destinationFile string) error
+
 	// Download Function Code
 	// @param destinationFile
 	//        file where data should be downloaded to
@@ -75,52 +111,130 @@ type Functions interface {
 	//        Function name
 	DownloadFunctionByNs(destinationFile, tenant, namespace, function string) error
 
-	// StartFunction start all function instances
+	// Download Function Code
+	// @param ctx
+	//        context used for the request
+	// @param destinationFile
+	//        file where data should be downloaded to
+	// @param tenant
+	//        Tenant name
+	// @param namespace
+	//        Namespace name
+	// @param function
+	//        Function name
+	DownloadFunctionByNsWithContext(ctx context.Context, destinationFile, tenant, namespace, function string) error
+
+	// StartFunction starts all function instances
 	StartFunction(tenant, namespace, name string) error
 
-	// StartFunctionWithID start function instance
+	// StartFunctionWithContext starts all function instances
+	StartFunctionWithContext(ctx context.Context, tenant, namespace, name string) error
+
+	// StartFunctionWithID starts function instance
 	StartFunctionWithID(tenant, namespace, name string, instanceID int) error
 
+	// StartFunctionWithIDWithContext starts function instance
+	StartFunctionWithIDWithContext(ctx context.Context, tenant, namespace, name string, instanceID int) error
+
 	// RestartFunction restart all function instances
 	RestartFunction(tenant, namespace, name string) error
 
+	// RestartFunctionWithContext restart all function instances
+	RestartFunctionWithContext(ctx context.Context, tenant, namespace, name string) error
+
 	// RestartFunctionWithID restart function instance
 	RestartFunctionWithID(tenant, namespace, name string, instanceID int) error
 
+	// RestartFunctionWithIDWithContext restart function instance
+	RestartFunctionWithIDWithContext(ctx context.Context, tenant, namespace, name string, instanceID int) error
+
 	// GetFunctions returns the list of functions
 	GetFunctions(tenant, namespace string) ([]string, error)
 
+	// GetFunctionsWithContext returns the list of functions
+	GetFunctionsWithContext(ctx context.Context, tenant, namespace string) ([]string, error)
+
 	// GetFunction returns the configuration for the specified function
 	GetFunction(tenant, namespace, name string) (utils.FunctionConfig, error)
 
+	// GetFunctionWithContext returns the configuration for the specified function
+	GetFunctionWithContext(ctx context.Context, tenant, namespace, name string) (utils.FunctionConfig, error)
+
 	// GetFunctionStatus returns the current status of a function
 	GetFunctionStatus(tenant, namespace, name string) (utils.FunctionStatus, error)
 
+	// GetFunctionStatusWithContext returns the current status of a function
+	GetFunctionStatusWithContext(ctx context.Context, tenant, namespace, name string) (utils.FunctionStatus, error)
+
 	// GetFunctionStatusWithInstanceID returns the current status of a function instance
 	GetFunctionStatusWithInstanceID(tenant, namespace, name string, instanceID int) (
 		utils.FunctionInstanceStatusData, error)
 
+	// GetFunctionStatusWithInstanceIDWithContext returns the current status of a function instance
+	GetFunctionStatusWithInstanceIDWithContext(ctx context.Context, tenant, namespace, name string, instanceID int) (
+		utils.FunctionInstanceStatusData, error)
+
 	// GetFunctionStats returns the current stats of a function
 	GetFunctionStats(tenant, namespace, name string) (utils.FunctionStats, error)
 
+	// GetFunctionStatsWithContext returns the current stats of a function
+	GetFunctionStatsWithContext(ctx context.Context, tenant, namespace, name string) (utils.FunctionStats, error)
+
 	// GetFunctionStatsWithInstanceID gets the current stats of a function instance
 	GetFunctionStatsWithInstanceID(tenant, namespace, name string, instanceID int) (utils.FunctionInstanceStatsData, error)
 
-	// GetFunctionState fetch the current state associated with a Pulsar Function
+	// GetFunctionStatsWithInstanceIDWithContext gets the current stats of a function instance
+	GetFunctionStatsWithInstanceIDWithContext(
+		ctx context.Context,
+		tenant,
+		namespace,
+		name string,
+		instanceID int,
+	) (utils.FunctionInstanceStatsData, error)
+
+	// GetFunctionState fetches the current state associated with a Pulsar Function
 	//
 	// Response Example:
 	// 		{ "value : 12, version : 2"}
 	GetFunctionState(tenant, namespace, name, key string) (utils.FunctionState, error)
 
+	// GetFunctionStateWithContext fetches the current state associated with a Pulsar Function
+	//
+	// Response Example:
+	// 		{ "value : 12, version : 2"}
+	GetFunctionStateWithContext(ctx context.Context, tenant, namespace, name, key string) (utils.FunctionState, error)
+
 	// PutFunctionState puts the given state associated with a Pulsar Function
 	PutFunctionState(tenant, namespace, name string, state utils.FunctionState) error
 
+	// PutFunctionStateWithContext puts the given state associated with a Pulsar Function
+	PutFunctionStateWithContext(ctx context.Context, tenant, namespace, name string, state utils.FunctionState) error
+
 	// TriggerFunction triggers the function by writing to the input topic
 	TriggerFunction(tenant, namespace, name, topic, triggerValue, triggerFile string) (string, error)
 
+	// TriggerFunctionWithContext triggers the function by writing to the input topic
+	TriggerFunctionWithContext(
+		ctx context.Context,
+		tenant,
+		namespace,
+		name,
+		topic,
+		triggerValue,
+		triggerFile string,
+	) (string, error)
+
 	// UpdateFunction updates the configuration for a function.
 	UpdateFunction(functionConfig *utils.FunctionConfig, fileName string, updateOptions *utils.UpdateOptions) error
 
+	// UpdateFunctionWithContext updates the configuration for a function.
+	UpdateFunctionWithContext(
+		ctx context.Context,
+		functionConfig *utils.FunctionConfig,
+		fileName string,
+		updateOptions *utils.UpdateOptions,
+	) error
+
 	// UpdateFunctionWithURL updates the configuration for a function.
 	//
 	// Update a function by providing url from which fun-pkg can be downloaded. supported url: http/file
@@ -129,8 +243,24 @@ type Functions interface {
 	// Http: http://www.repo.com/fileName.jar
 	UpdateFunctionWithURL(functionConfig *utils.FunctionConfig, pkgURL string, updateOptions *utils.UpdateOptions) error
 
+	// UpdateFunctionWithURLWithContext updates the configuration for a function.
+	//
+	// Update a function by providing url from which fun-pkg can be downloaded. supported url: http/file
+	// eg:
+	// File: file:/dir/fileName.jar
+	// Http: http://www.repo.com/fileName.jar
+	UpdateFunctionWithURLWithContext(
+		ctx context.Context,
+		functionConfig *utils.FunctionConfig,
+		pkgURL string,
+		updateOptions *utils.UpdateOptions,
+	) error
+
 	// Upload function to Pulsar
 	Upload(sourceFile, path string) error
+
+	// UploadWithContext function to Pulsar
+	UploadWithContext(ctx context.Context, sourceFile, path string) error
 }
 
 type functions struct {
@@ -161,6 +291,10 @@ func (f *functions) createTextFromFiled(w *multipart.Writer, value string) (io.W
 }
 
 func (f *functions) CreateFunc(funcConf *utils.FunctionConfig, fileName string) error {
+	return f.CreateFuncWithContext(context.Background(), funcConf, fileName)
+}
+
+func (f *functions) CreateFuncWithContext(ctx context.Context, funcConf *utils.FunctionConfig, fileName string) error {
 	endpoint := f.pulsar.endpoint(f.basePath, funcConf.Tenant, funcConf.Namespace, funcConf.Name)
 
 	// buffer to store our request as bytes
@@ -211,7 +345,7 @@ func (f *functions) CreateFunc(funcConf *utils.FunctionConfig, fileName string)
 	}
 
 	contentType := multiPartWriter.FormDataContentType()
-	err = f.pulsar.Client.PostWithMultiPart(endpoint, nil, bodyBuf, contentType)
+	err = f.pulsar.Client.PostWithMultiPartWithContext(ctx, endpoint, nil, bodyBuf, contentType)
 	if err != nil {
 		return err
 	}
@@ -220,6 +354,14 @@ func (f *functions) CreateFunc(funcConf *utils.FunctionConfig, fileName string)
 }
 
 func (f *functions) CreateFuncWithURL(funcConf *utils.FunctionConfig, pkgURL string) error {
+	return f.CreateFuncWithURLWithContext(context.Background(), funcConf, pkgURL)
+}
+
+func (f *functions) CreateFuncWithURLWithContext(
+	ctx context.Context,
+	funcConf *utils.FunctionConfig,
+	pkgURL string,
+) error {
 	endpoint := f.pulsar.endpoint(f.basePath, funcConf.Tenant, funcConf.Namespace, funcConf.Name)
 	// buffer to store our request as bytes
 	bodyBuf := bytes.NewBufferString("")
@@ -256,7 +398,7 @@ func (f *functions) CreateFuncWithURL(funcConf *utils.FunctionConfig, pkgURL str
 	}
 
 	contentType := multiPartWriter.FormDataContentType()
-	err = f.pulsar.Client.PostWithMultiPart(endpoint, nil, bodyBuf, contentType)
+	err = f.pulsar.Client.PostWithMultiPartWithContext(ctx, endpoint, nil, bodyBuf, contentType)
 	if err != nil {
 		return err
 	}
@@ -265,23 +407,45 @@ func (f *functions) CreateFuncWithURL(funcConf *utils.FunctionConfig, pkgURL str
 }
 
 func (f *functions) StopFunction(tenant, namespace, name string) error {
+	return f.StopFunctionWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) StopFunctionWithContext(ctx context.Context, tenant, namespace, name string) error {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	return f.pulsar.Client.Post(endpoint+"/stop", nil)
+	return f.pulsar.Client.PostWithContext(ctx, endpoint+"/stop", nil)
 }
 
 func (f *functions) StopFunctionWithID(tenant, namespace, name string, instanceID int) error {
+	return f.StopFunctionWithIDWithContext(context.Background(), tenant, namespace, name, instanceID)
+}
+
+func (f *functions) StopFunctionWithIDWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+	instanceID int,
+) error {
 	id := fmt.Sprintf("%d", instanceID)
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, id)
 
-	return f.pulsar.Client.Post(endpoint+"/stop", nil)
+	return f.pulsar.Client.PostWithContext(ctx, endpoint+"/stop", nil)
 }
 
 func (f *functions) DeleteFunction(tenant, namespace, name string) error {
+	return f.DeleteFunctionWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) DeleteFunctionWithContext(ctx context.Context, tenant, namespace, name string) error {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	return f.pulsar.Client.Delete(endpoint)
+	return f.pulsar.Client.DeleteWithContext(ctx, endpoint)
 }
 
 func (f *functions) DownloadFunction(path, destinationFile string) error {
+	return f.DownloadFunctionWithContext(context.Background(), path, destinationFile)
+}
+
+func (f *functions) DownloadFunctionWithContext(ctx context.Context, path, destinationFile string) error {
 	endpoint := f.pulsar.endpoint(f.basePath, "download")
 	_, err := os.Open(destinationFile)
 	if err != nil {
@@ -298,7 +462,7 @@ func (f *functions) DownloadFunction(path, destinationFile string) error {
 	tmpMap := make(map[string]string)
 	tmpMap["path"] = path
 
-	_, err = f.pulsar.Client.GetWithOptions(endpoint, nil, tmpMap, false, file)
+	_, err = f.pulsar.Client.GetWithOptionsWithContext(ctx, endpoint, nil, tmpMap, false, file)
 	if err != nil {
 		return err
 	}
@@ -306,6 +470,16 @@ func (f *functions) DownloadFunction(path, destinationFile string) error {
 }
 
 func (f *functions) DownloadFunctionByNs(destinationFile, tenant, namespace, function string) error {
+	return f.DownloadFunctionByNsWithContext(context.Background(), destinationFile, tenant, namespace, function)
+}
+
+func (f *functions) DownloadFunctionByNsWithContext(
+	ctx context.Context,
+	destinationFile,
+	tenant,
+	namespace,
+	function string,
+) error {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, function, "download")
 	_, err := os.Open(destinationFile)
 	if err != nil {
@@ -319,7 +493,7 @@ func (f *functions) DownloadFunctionByNs(destinationFile, tenant, namespace, fun
 		return err
 	}
 
-	_, err = f.pulsar.Client.GetWithOptions(endpoint, nil, nil, false, file)
+	_, err = f.pulsar.Client.GetWithOptionsWithContext(ctx, endpoint, nil, nil, false, file)
 	if err != nil {
 		return err
 	}
@@ -328,45 +502,95 @@ func (f *functions) DownloadFunctionByNs(destinationFile, tenant, namespace, fun
 }
 
 func (f *functions) StartFunction(tenant, namespace, name string) error {
+	return f.StartFunctionWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) StartFunctionWithContext(ctx context.Context, tenant, namespace, name string) error {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	return f.pulsar.Client.Post(endpoint+"/start", nil)
+	return f.pulsar.Client.PostWithContext(ctx, endpoint+"/start", nil)
 }
 
 func (f *functions) StartFunctionWithID(tenant, namespace, name string, instanceID int) error {
+	return f.StartFunctionWithIDWithContext(context.Background(), tenant, namespace, name, instanceID)
+}
+
+func (f *functions) StartFunctionWithIDWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+	instanceID int,
+) error {
 	id := fmt.Sprintf("%d", instanceID)
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, id)
 
-	return f.pulsar.Client.Post(endpoint+"/start", nil)
+	return f.pulsar.Client.PostWithContext(ctx, endpoint+"/start", nil)
 }
 
 func (f *functions) RestartFunction(tenant, namespace, name string) error {
+	return f.RestartFunctionWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) RestartFunctionWithContext(ctx context.Context, tenant, namespace, name string) error {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	return f.pulsar.Client.Post(endpoint+"/restart", nil)
+	return f.pulsar.Client.PostWithContext(ctx, endpoint+"/restart", nil)
 }
 
 func (f *functions) RestartFunctionWithID(tenant, namespace, name string, instanceID int) error {
+	return f.RestartFunctionWithIDWithContext(context.Background(), tenant, namespace, name, instanceID)
+}
+
+func (f *functions) RestartFunctionWithIDWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+	instanceID int,
+) error {
 	id := fmt.Sprintf("%d", instanceID)
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, id)
 
-	return f.pulsar.Client.Post(endpoint+"/restart", nil)
+	return f.pulsar.Client.PostWithContext(ctx, endpoint+"/restart", nil)
 }
 
 func (f *functions) GetFunctions(tenant, namespace string) ([]string, error) {
+	return f.GetFunctionsWithContext(context.Background(), tenant, namespace)
+}
+
+func (f *functions) GetFunctionsWithContext(ctx context.Context, tenant, namespace string) ([]string, error) {
 	var functions []string
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace)
-	err := f.pulsar.Client.Get(endpoint, &functions)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint, &functions)
 	return functions, err
 }
 
 func (f *functions) GetFunction(tenant, namespace, name string) (utils.FunctionConfig, error) {
+	return f.GetFunctionWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) GetFunctionWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+) (utils.FunctionConfig, error) {
 	var functionConfig utils.FunctionConfig
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	err := f.pulsar.Client.Get(endpoint, &functionConfig)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint, &functionConfig)
 	return functionConfig, err
 }
 
 func (f *functions) UpdateFunction(functionConfig *utils.FunctionConfig, fileName string,
 	updateOptions *utils.UpdateOptions) error {
+	return f.UpdateFunctionWithContext(context.Background(), functionConfig, fileName, updateOptions)
+}
+
+func (f *functions) UpdateFunctionWithContext(
+	ctx context.Context,
+	functionConfig *utils.FunctionConfig,
+	fileName string,
+	updateOptions *utils.UpdateOptions,
+) error {
 	endpoint := f.pulsar.endpoint(f.basePath, functionConfig.Tenant, functionConfig.Namespace, functionConfig.Name)
 	// buffer to store our request as bytes
 	bodyBuf := bytes.NewBufferString("")
@@ -433,7 +657,7 @@ func (f *functions) UpdateFunction(functionConfig *utils.FunctionConfig, fileNam
 	}
 
 	contentType := multiPartWriter.FormDataContentType()
-	err = f.pulsar.Client.PutWithMultiPart(endpoint, bodyBuf, contentType)
+	err = f.pulsar.Client.PutWithMultiPartWithContext(ctx, endpoint, bodyBuf, contentType)
 	if err != nil {
 		return err
 	}
@@ -443,6 +667,15 @@ func (f *functions) UpdateFunction(functionConfig *utils.FunctionConfig, fileNam
 
 func (f *functions) UpdateFunctionWithURL(functionConfig *utils.FunctionConfig, pkgURL string,
 	updateOptions *utils.UpdateOptions) error {
+	return f.UpdateFunctionWithURLWithContext(context.Background(), functionConfig, pkgURL, updateOptions)
+}
+
+func (f *functions) UpdateFunctionWithURLWithContext(
+	ctx context.Context,
+	functionConfig *utils.FunctionConfig,
+	pkgURL string,
+	updateOptions *utils.UpdateOptions,
+) error {
 	endpoint := f.pulsar.endpoint(f.basePath, functionConfig.Tenant, functionConfig.Namespace, functionConfig.Name)
 	// buffer to store our request as bytes
 	bodyBuf := bytes.NewBufferString("")
@@ -498,7 +731,7 @@ func (f *functions) UpdateFunctionWithURL(functionConfig *utils.FunctionConfig,
 	}
 
 	contentType := multiPartWriter.FormDataContentType()
-	err = f.pulsar.Client.PutWithMultiPart(endpoint, bodyBuf, contentType)
+	err = f.pulsar.Client.PutWithMultiPartWithContext(ctx, endpoint, bodyBuf, contentType)
 	if err != nil {
 		return err
 	}
@@ -507,45 +740,93 @@ func (f *functions) UpdateFunctionWithURL(functionConfig *utils.FunctionConfig,
 }
 
 func (f *functions) GetFunctionStatus(tenant, namespace, name string) (utils.FunctionStatus, error) {
+	return f.GetFunctionStatusWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) GetFunctionStatusWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+) (utils.FunctionStatus, error) {
 	var functionStatus utils.FunctionStatus
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	err := f.pulsar.Client.Get(endpoint+"/status", &functionStatus)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint+"/status", &functionStatus)
 	return functionStatus, err
 }
 
 func (f *functions) GetFunctionStatusWithInstanceID(tenant, namespace, name string,
+	instanceID int) (utils.FunctionInstanceStatusData, error) {
+	return f.GetFunctionStatusWithInstanceIDWithContext(context.Background(), tenant, namespace, name, instanceID)
+}
+
+func (f *functions) GetFunctionStatusWithInstanceIDWithContext(ctx context.Context, tenant, namespace, name string,
 	instanceID int) (utils.FunctionInstanceStatusData, error) {
 	var functionInstanceStatusData utils.FunctionInstanceStatusData
 	id := fmt.Sprintf("%d", instanceID)
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, id)
-	err := f.pulsar.Client.Get(endpoint+"/status", &functionInstanceStatusData)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint+"/status", &functionInstanceStatusData)
 	return functionInstanceStatusData, err
 }
 
 func (f *functions) GetFunctionStats(tenant, namespace, name string) (utils.FunctionStats, error) {
+	return f.GetFunctionStatsWithContext(context.Background(), tenant, namespace, name)
+}
+
+func (f *functions) GetFunctionStatsWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+) (utils.FunctionStats, error) {
 	var functionStats utils.FunctionStats
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name)
-	err := f.pulsar.Client.Get(endpoint+"/stats", &functionStats)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint+"/stats", &functionStats)
 	return functionStats, err
 }
 
 func (f *functions) GetFunctionStatsWithInstanceID(tenant, namespace, name string,
+	instanceID int) (utils.FunctionInstanceStatsData, error) {
+	return f.GetFunctionStatsWithInstanceIDWithContext(context.Background(), tenant, namespace, name, instanceID)
+}
+
+func (f *functions) GetFunctionStatsWithInstanceIDWithContext(ctx context.Context, tenant, namespace, name string,
 	instanceID int) (utils.FunctionInstanceStatsData, error) {
 	var functionInstanceStatsData utils.FunctionInstanceStatsData
 	id := fmt.Sprintf("%d", instanceID)
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, id)
-	err := f.pulsar.Client.Get(endpoint+"/stats", &functionInstanceStatsData)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint+"/stats", &functionInstanceStatsData)
 	return functionInstanceStatsData, err
 }
 
 func (f *functions) GetFunctionState(tenant, namespace, name, key string) (utils.FunctionState, error) {
+	return f.GetFunctionStateWithContext(context.Background(), tenant, namespace, name, key)
+}
+
+func (f *functions) GetFunctionStateWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name,
+	key string,
+) (utils.FunctionState, error) {
 	var functionState utils.FunctionState
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, "state", key)
-	err := f.pulsar.Client.Get(endpoint, &functionState)
+	err := f.pulsar.Client.GetWithContext(ctx, endpoint, &functionState)
 	return functionState, err
 }
 
 func (f *functions) PutFunctionState(tenant, namespace, name string, state utils.FunctionState) error {
+	return f.PutFunctionStateWithContext(context.Background(), tenant, namespace, name, state)
+}
+
+func (f *functions) PutFunctionStateWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name string,
+	state utils.FunctionState,
+) error {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, "state", state.Key)
 
 	// buffer to store our request as bytes
@@ -578,7 +859,7 @@ func (f *functions) PutFunctionState(tenant, namespace, name string, state utils
 
 	contentType := multiPartWriter.FormDataContentType()
 
-	err = f.pulsar.Client.PostWithMultiPart(endpoint, nil, bodyBuf, contentType)
+	err = f.pulsar.Client.PostWithMultiPartWithContext(ctx, endpoint, nil, bodyBuf, contentType)
 
 	if err != nil {
 		return err
@@ -588,6 +869,18 @@ func (f *functions) PutFunctionState(tenant, namespace, name string, state utils
 }
 
 func (f *functions) TriggerFunction(tenant, namespace, name, topic, triggerValue, triggerFile string) (string, error) {
+	return f.TriggerFunctionWithContext(context.Background(), tenant, namespace, name, topic, triggerValue, triggerFile)
+}
+
+func (f *functions) TriggerFunctionWithContext(
+	ctx context.Context,
+	tenant,
+	namespace,
+	name,
+	topic,
+	triggerValue,
+	triggerFile string,
+) (string, error) {
 	endpoint := f.pulsar.endpoint(f.basePath, tenant, namespace, name, "trigger")
 
 	// buffer to store our request as bytes
@@ -647,7 +940,7 @@ func (f *functions) TriggerFunction(tenant, namespace, name, topic, triggerValue
 
 	contentType := multiPartWriter.FormDataContentType()
 	var str string
-	err := f.pulsar.Client.PostWithMultiPart(endpoint, &str, bodyBuf, contentType)
+	err := f.pulsar.Client.PostWithMultiPartWithContext(ctx, endpoint, &str, bodyBuf, contentType)
 	if err != nil {
 		return "", err
 	}
@@ -656,6 +949,10 @@ func (f *functions) TriggerFunction(tenant, namespace, name, topic, triggerValue
 }
 
 func (f *functions) Upload(sourceFile, path string) error {
+	return f.UploadWithContext(context.Background(), sourceFile, path)
+}
+
+func (f *functions) UploadWithContext(ctx context.Context, sourceFile, path string) error {
 	if strings.TrimSpace(sourceFile) == "" && strings.TrimSpace(path) == "" {
 		return fmt.Errorf("source file or path is empty")
 	}
@@ -682,5 +979,5 @@ func (f *functions) Upload(sourceFile, path string) error {
 	if err != nil {
 		return err
 	}
-	return f.pulsar.Client.PostWithMultiPart(endpoint, nil, &b, w.FormDataContentType())
+	return f.pulsar.Client.PostWithMultiPartWithContext(ctx, endpoint, nil, &b, w.FormDataContentType())
 }
