@@ -23,6 +23,8 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
 	"github.com/apache/pulsar-client-go/pulsar/backoff"
 
 	"github.com/apache/pulsar-client-go/pulsar/crypto"
@@ -1029,8 +1031,8 @@ func TestReaderHasNextRetryFailed(t *testing.T) {
 			close(req.doneCh)
 		}
 	}()
-	minTimer := time.NewTimer(1 * time.Second) // Timer to check if r.HasNext() blocked for at least 1s
-	maxTimer := time.NewTimer(3 * time.Second) // Timer to ensure r.HasNext() doesn't block for more than 3s
+	maxTimer := time.NewTimer(8 * time.Second) // Timer to ensure r.HasNext() doesn't block for more than 3s
+	startTime := time.Now().UnixMilli()
 	done := make(chan bool)
 	go func() {
 		assert.False(t, r.HasNext())
@@ -1041,8 +1043,10 @@ func TestReaderHasNextRetryFailed(t *testing.T) {
 	case <-maxTimer.C:
 		t.Fatal("r.HasNext() blocked for more than 3s")
 	case <-done:
-		assert.False(t, minTimer.Stop(), "r.HasNext() did not block for at least 1s")
-		assert.True(t, maxTimer.Stop())
+		now := time.Now().UnixMilli()
+		if now-startTime < 1000 {
+			t.Fatal("r.HasNext() blocked for less than 1s")
+		}
 	}
 
 }
@@ -1237,3 +1241,60 @@ func TestReaderWithSeekByTime(t *testing.T) {
 		})
 	}
 }
+
+func TestReaderReadFromLatest(t *testing.T) {
+	topic := newTopicName()
+	client, err := NewClient(ClientOptions{
+		URL: lookupURL,
+	})
+
+	require.NoError(t, err)
+	defer client.Close()
+
+	r, err := client.CreateReader(ReaderOptions{
+		Topic:          topic,
+		StartMessageID: LatestMessageID(),
+	})
+	require.NoError(t, err)
+	defer r.Close()
+
+	p, err := client.CreateProducer(ProducerOptions{
+		Topic: topic,
+	})
+	require.NoError(t, err)
+	defer p.Close()
+
+	// Send messages
+	for i := 0; i < 10; i++ {
+		msg := &ProducerMessage{
+			Key:     "key",
+			Payload: []byte(fmt.Sprintf("message-%d", i)),
+		}
+		id, err := p.Send(context.Background(), msg)
+		require.NoError(t, err)
+		require.NotNil(t, id)
+	}
+
+	// Read and verify messages
+	for i := 0; i < 10; i++ {
+		ctx, cancel := context.WithTimeout(context.Background(), 1*time.Second)
+		msg, err := r.Next(ctx)
+		cancel()
+		require.NoError(t, err)
+		require.NotNil(t, msg)
+
+		// Verify message key
+		require.Equal(t, "key", msg.Key())
+
+		// Verify message payload
+		expectedPayload := fmt.Sprintf("message-%d", i)
+		require.Equal(t, []byte(expectedPayload), msg.Payload())
+	}
+
+	// Verify no more messages
+	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Second)
+	defer cancel()
+	msg, err := r.Next(ctx)
+	require.Error(t, err)
+	require.Nil(t, msg)
+}
